/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"

using namespace std;
using namespace FD;
   
class CvfdAdd;
   
DECLARE_NODE(CvfdAdd)
/*Node
 *
 * @name CvfdAdd
 * @category FDOPENCV_ORG:Arithmetic Logic Comparison
 * @description Adds one image to another one:
 * dst(I)=src1(I)+src2(I).
 * The images must have the same type and the same size.
 *
 * @input_name IMAGE1
 * @input_description Input image1.
 * @input_type CvfdImage
 *
 * @input_name IMAGE2
 * @input_description Input image2.
 * @input_type CvfdImage
 *
 * @output_name OUTPUT
 * @output_description Output image.
 * @output_type CvfdImage
 *
 *
 END*/
   
   
class CvfdAdd : public BufferedNode 
{
  //Input ID            
  int m_image1ID;
  int m_image2ID;
  //Output ID
  int m_outputID;
      
public:
  CvfdAdd(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    //add inputs
    m_image1ID = addInput("IMAGE1");
    m_image2ID = addInput("IMAGE2");
    //add outputs
    m_outputID = addOutput("OUTPUT");
  }      
      
  void calculate(int output_id, int count, Buffer &out)
  {
    //Read the inputs
    RCPtr<CvfdImage> image1Ptr = getInput(m_image1ID,count);   
    RCPtr<CvfdImage> image2Ptr = getInput(m_image2ID,count);
    //Handle
    CvfdImage* image;
    int status = cvGetErrMode();
    cvSetErrMode( CV_ErrModeSilent );
    image = new CvfdImage(image1Ptr->getImage());
    __BEGIN__;
    OPENCV_CALL(cvAdd(image1Ptr->getImage(), image2Ptr->getImage(), image->getImage()));          
    __END__;
    cvSetErrMode( status );
         
    if( cvGetErrStatus() != CV_StsOk  )
      {
	throw new GeneralException("OPENCV - Error to add the images: " +  CCHAR(cvErrorStr( cvGetErrStatus() )),__FILE__,__LINE__);
      }
         
    out[count] = ObjectRef(image);
  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdAdd)
};
